import { Menu } from 'electron';
import { Color } from 'vs/base/common/color';
import { Dimension } from 'vs/base/browser/dom';
import { TitlebarOptions } from './types/titlebar-options';
import { MenuBar } from 'vs/base/browser/ui/menu/menubar';
import { Disposable } from 'vs/base/common/lifecycle';
import { Event } from 'vs/base/common/event';
import { RunOnceScheduler } from 'vs/base/common/async';
export declare class Titlebar extends Disposable {
    _titlebar: HTMLElement;
    _dragRegion: HTMLElement;
    _windowIcon: HTMLImageElement;
    _title: HTMLElement;
    _menubarContainer: HTMLElement;
    _windowControls: HTMLElement;
    _container: HTMLElement;
    _isInactive?: boolean;
    _menubar?: MenuBar;
    _options: TitlebarOptions;
    _windowControlIcons: {
        minimize: HTMLElement;
        maximize: HTMLElement;
        close: HTMLElement;
    };
    _resizer: {
        top: HTMLElement;
        left: HTMLElement;
    };
    _currentMenu: Menu | undefined;
    _defaultOptions: TitlebarOptions;
    _platformIcons: {
        [key: string]: string;
    };
    private reinstallDisposables;
    private readonly _onVisibilityChange;
    private readonly _onFocusStateChange;
    protected menuUpdater: RunOnceScheduler;
    constructor(titlebarOptions?: TitlebarOptions);
    /**
     * Load the icons for the window controls
     */
    _loadIcons(): void;
    /**
     * Load the background color for the titlebar
     */
    _loadBackgroundColor(): void;
    /**
     * Setup the container for the titlebar
     */
    _setupTitlebar(): void;
    /**
     * Setup the container after the titlebar
     */
    _setupContainer(): void;
    /**
     * Load events for the titlebar
     */
    _loadEvents(): void;
    _closeMenu: () => void;
    _setupIcon(): void;
    _setupMenubar(): void;
    _setupTitle(): void;
    _setIconSize(size?: number): void;
    _createControl(control: HTMLElement, enabled: boolean | undefined, title: string, icon: string, className: string): void;
    _createControls(): void;
    _onBlur(): void;
    _onFocus(): void;
    _onMenubarVisibilityChanged(visible: boolean): void;
    _onMenubarFocusChanged(focused: boolean): void;
    _onDidChangeMaximized(isMaximized: boolean): void;
    _updateStyles(): void;
    protected doUpdateMenubar(firstTime: boolean): void;
    /**
     * Update the default menu or set a new menu.
     * @param menu The menu.
     * @param firstTime If it is the first time to set the menu.
     */
    private updateMenu;
    private getMenuBarOptions;
    private updateMenubar;
    private onDidVisibilityChange;
    /**
     * Update title bar styles based on focus state.
     * @param focus focus state of the window
     */
    onWindowFocus(focus: boolean): void;
    /**
     * Update the full screen state and hide or show the title bar.
     * @param fullscreen Fullscreen state of the window
     */
    onWindowFullScreen(fullscreen: boolean): void;
    /**
     * Update the background color of the title bar
     * @param backgroundColor The color for the background
     */
    updateBackground(backgroundColor: Color): Titlebar;
    /**
     * Update the item background color of the menubar
     * @param itemBGColor The color for the item background
     */
    updateItemBGColor(itemBGColor: Color): Titlebar;
    /**
     * Update the title of the title bar.
     * You can use this method if change the content of `<title>` tag on your html.
     * @param title The title of the title bar and document.
     */
    updateTitle(title: string): void;
    /**
     * It method set new icon to title-bar-icon of title-bar.
     * @param path path to icon
     */
    updateIcon(path?: string): void;
    /**
     * Update the menu from Menu.getApplicationMenu()
     */
    refreshMenu(): Promise<void>;
    /**
     * Update the position of menubar.
     * @param menuPosition The position of the menu `left`, `right` or `bottom`.
     */
    updateMenuPosition(menuPosition: "left" | "right" | "bottom"): Titlebar;
    /**
     * Horizontal alignment of the title.
     * @param side `left`, `center` or `right`.
     */
    updateTitleAlignment(side: "left" | "center" | "right"): Titlebar;
    protected adjustTitleMarginToCenter(): void;
    /**
     * Remove the titlebar, menubar and all methods.
     */
    dispose(): void;
    get onVisibilityChange(): Event<boolean>;
    get onFocusStateChange(): Event<boolean>;
    registerListeners(): void;
    getMenubarItemsDimensions(): Dimension;
}
/**
 * Handles mnemonics for menu items. Depending on OS:
 * - Windows: Supported via & character (replace && with &)
 * -   Linux: Supported via & character (replace && with &)
 * -   macOS: Unsupported (replace && with empty string)
 */
export declare function mnemonicMenuLabel(label: string, forceDisableMnemonics?: boolean): string;
